#! /bin/bash

# Author: GNot
# URL: http://www.g-loaded.eu/2005/12/15/get-my-kernel-headers-script/
# Version: 0.4beta
# License: GPL
#
# Usage: get_my_kernel_headers.sh
#
# This script fetches the fedora kernel SRPM,
# does all things outlined in my blog post:
# http://www.g-loaded.eu/2005/12/14/the-complete-fedora-kernel-headers/
# This is a stupid description. I'll edit when I have enough time...
# Feel free to edit it as you like

# Requires: wget, rpm, rpmbuild, sed


# CONFIG
# EDIT BEGIN (Note: Do not add trailing slash in directory paths)
DownURL="http://download.fedora.redhat.com/pub/fedora/linux/core/updates/5/SRPMS/"
WorkingBase="/usr/src/redhat"
KeepSRPM="1"
# EDIT END


######## INITIAL CHECKS #########
echo -e "\n************** INITIAL CHECKS **************"
# Check if the Module.symvers from the proper kernel-devel package exists
echo "Checking if Module.symvers from kernel-devel-$(uname -r)-$(arch) exists:"
if [ -f /usr/src/kernels/$(uname -r)-$(arch)/Module.symvers ]; then
	echo -e "\t/usr/src/kernels/$(uname -r)-$(arch)/Module.symvers file exists"
else
	echo -e "\tkernel-devel is not installed. Please install. The Module.symvers file is needed."
	exit 1
fi


# Check if is root
echo "Checking if user is root"
# Comment the following line if you want to allow root to perform these actions - NOT RECOMMENDED
[ $(id -u) -eq 0 ] && echo -e "\tYou should NOT run this script as root." && exit 1
echo -e "\tUser is not root - Continuing..."

# Kernel related Vars
KernVerRel="$(uname -r)"
KernVer="${KernVerRel%%-*}"
KernRel="${KernVerRel##*-}"

echo -e "\nKERNEL INFO"
echo "Kernel: $KernVerRel"
echo "Version: $KernVer"
echo "Release: $KernRel"
echo "Architecture: $(arch)"


echo -e "\n************** RUNNING SCRIPT **************"

# Download the SRPM for the current kernel in WorkingBase
echo -e "\nSTEP 1: Downloading kernel SRPM"
cd "$WorkingBase/SRPMS/"
if [ -f "$WorkingBase/SRPMS/kernel-$KernVerRel.src.rpm" ]; then
	echo -e "\tFile already exists in $WorkingBase/SRPMS/ - No Downloading"
else
	wget "$DownURL/kernel-$KernVerRel.src.rpm"
	#wget -S "$DownURL/kernel-$KernVerRel.src.rpm"
fi
cd "$WorkingBase/"


# Install the kernel SRPM
echo -e "\nSTEP 2: Kernel SRPM Installation (aka extracting the contents)"
rpm -ivh SRPMS/kernel-$KernVerRel.src.rpm
echo -e "\tKernel SRPM installed"


# Execute %prep stage
echo -e "\nSTEP 3: Execute %prep stage - Extract sources and apply patches"
echo -e "\tThis step will take a couple of minutes to complete..."
rpmbuild -bp --target $(arch) --rmsource --rmspec --quiet SPECS/kernel-2.6.spec &> /dev/null
echo -e "\tSources extracted and patches applied"


# Copy config
echo -e "\nSTEP 4: Copying configuration"
cd "$WorkingBase/BUILD/kernel-$KernVer/linux-$KernVer.$(arch)"
cp -f configs/kernel-$KernVer-$(arch).config .config
echo -e "\tKernel configuration file copied"


# Fill proper EXTRAVERSION in the Makefile
echo -e "\nSTEP 5: Filling proper EXTRAVERSION in the Makefile"
sed -i 's/^EXTRAVERSION[ ]\=[ ]\-prep$/EXTRAVERSION \= \-'$KernRel'/' Makefile
echo -e "\tEXTRAVERSION set"


# Make oldconfig
echo -e "\nSTEP 6: Make OldConfig"
make oldconfig &> /dev/null
echo -e "\tDone"


# Copy the Module.symvers file
echo -e "\nSTEP 7: Copy the Module.symvers file from kernel-devel"
cp /usr/src/kernels/$(uname -r)-$(arch)/Module.symvers .
echo -e "\tDone"


# Final Source Preparations
echo -e "\nSTEP 8: Final preparations"
make prepare scripts &> /dev/null
echo -e "\tDone"


# Directories
echo -e "\nSTEP 9: Proper directory"
cd "$WorkingBase/BUILD/"
mv "kernel-$KernVer/linux-$KernVer.$(arch)" "kernel-$KernVerRel-$(arch)-full"
rm -fr "kernel-$KernVer"
[ $KeepSRPM -ne 1 ] && rm -f "$WorkingBase/SRPMS/kernel-$KernVerRel.src.rpm"
echo -e "\tDone"
echo -e "\nKernel Source Directory is: $WorkingBase/BUILD/kernel-$KernVerRel-full\n"


exit 0
